package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysParamMapper
import com.gxzc.zen.api.sys.model.SysParam
import com.gxzc.zen.api.sys.service.ISysParamService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.contants.PLATFORM
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.properties.PlatformProperties
import com.gxzc.zen.common.util.PlatformUtil
import com.gxzc.zen.common.util.RedisCacheUtil
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.CommandLineRunner
import org.springframework.stereotype.Service

/**
 * <p>
 * 系统参数表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Suppress("UNCHECKED_CAST")
@Service
class SysParamServiceImpl : ServiceImpl<SysParamMapper, SysParam>(), ISysParamService, CommandLineRunner {
    companion object {
        private val logger = LoggerFactory.getLogger(SysParamServiceImpl::class.java)
        const val CACHE_KEY_ALL = "param_all"
    }

    @Autowired
    private lateinit var platformProperties: PlatformProperties

    override fun run(vararg args: String?) {
        logger.debug("${this::class.simpleName} init.")
        getListCacheable()
    }

    override fun getListCacheable(): MutableList<SysParam> {
        val cached = RedisCacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysParam>
        if (cached != null) {
            return cached
        }
        val ret = baseMapper.selectWOLogic(null)
        if (PlatformUtil.getPlatform(platformProperties) == PLATFORM.SYSTEM) {
            RedisCacheUtil.put(CACHEKEYS.SYS, CACHE_KEY_ALL, ret)
        }
        return ret
    }

    override fun getList(enable: Boolean?): MutableList<SysParam> {
        return if (enable != null) {
            baseMapper.selectWOLogic(EntityWrapper<SysParam>().eq("enable", enable))
        } else {
            baseMapper.selectWOLogic(null)
        }
    }

    override fun getListByKey(key: String): MutableList<SysParam> {
        val allData = getListCacheable()
        return allData.filter {
            it.enable != null && it.enable!! && it.key == key
        }.toMutableList()
    }

    @ZenTransactional
    override fun modify(data: SysParam): SysParam {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            baseMapper.updateWOLogic(data, EntityWrapper<SysParam>().eq("id", data.id))
            // 更新缓存
            val cached = RedisCacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysParam>
            cached?.let {
                val idx = it.indexOfFirst { it.id == data.id }
                if (idx != -1) {
                    it[idx] = data
                }
                RedisCacheUtil.put(CACHEKEYS.SYS, CACHE_KEY_ALL, it)
            }
        }
        return data
    }

    override fun getOneByKey(key: String, value: String?, sort: Int?): SysParam? {
        val result = baseMapper.selectWOLogic(EntityWrapper<SysParam>().apply {
            eq("key", key)
            if (value.isNullOrEmpty()) {
                eq("value", value!!)
            }
            if (sort != null) {
                eq("sort", sort)
            }
            eq("enable", true)
        })
        return if (result.size > 0) {
            result.filter {
                it.sort == sort && it.value == value
            }[0]
        } else {
            null
        }
    }

    @ZenTransactional
    override fun insertCacheable(data: SysParam) {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            if (baseMapper.insert(data) == 0) {
                throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
            }
            val cached = RedisCacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysParam>
            cached?.let {
                it.add(data)
                RedisCacheUtil.put(CACHEKEYS.SYS, CACHE_KEY_ALL, it)
            }
        }
    }

    @ZenTransactional
    override fun physicalDeleteCacheable(id: Long) {
        if (PlatformUtil.getPlatform() == PLATFORM.SYSTEM) {
            if (baseMapper.physicalDelete(EntityWrapper<SysParam>().eq("id", id)) <= 0) {
                throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
            }
            //
            val cached = RedisCacheUtil.get(CACHEKEYS.SYS, CACHE_KEY_ALL)?.get() as? MutableList<SysParam>
            cached?.let {
                it.removeIf {
                    it.id == id
                }
                RedisCacheUtil.put(CACHEKEYS.SYS, CACHE_KEY_ALL, it)
            }
        }
    }
}
