package com.gxzc.zen.api.sys.service.impl

import com.gxzc.zen.api.sys.service.ISysPermissionService
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.util.RedisCacheUtil
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * <p>
 * 权限表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysPermissionServiceImpl : ISysPermissionService {


    @Autowired
    private lateinit var sysUserRoleService: ISysUserRoleService

    @Cacheable(value = [CACHEKEYS.USER_PERM], key = "'${CACHEKEYS.USER_PERM}:uid_' + #id")
    override fun getPermissionSetByUserId(id: Long): HashSet<String> {
        val roleList = sysUserRoleService.getUserRoleListByUserId(id)
        val permIds = linkedSetOf<String>()
        for (item in roleList) {
            item.perms?.split(',')?.toCollection(permIds)
        }
        return permIds
    }

    //    @CacheEvict(value = [CACHEKEYS.USER_PERM], key = "'${CACHEKEYS.USER_PERM}:uid_' + #id")
    override fun evictCache(id: Long) {
        RedisCacheUtil.evict(CACHEKEYS.USER_PERM, "${CACHEKEYS.USER_PERM}:uid_$id")
    }

    override fun evictCache(ids: Set<Long>) {
        ids.forEach {
            RedisCacheUtil.evict(CACHEKEYS.USER_PERM, "${CACHEKEYS.USER_PERM}:uid_$it")
        }
    }

    override fun clearCache() {
        RedisCacheUtil.clear(CACHEKEYS.USER_PERM)
    }

//    @PostConstruct
//    override fun initAllUserPermissionCache() {
//        val cache = cacheManager.getCache(CACHEKEYS.USER_PERM)
//        val userRoleMap = sysUserRoleService.getUserRoleList()
//        for (item in userRoleMap) {
//            val permIds = linkedSetOf<String>()
//            for (role in item.value) {
//                role.perms?.split(',')?.toCollection(permIds)
//            }
//            cache.put("uid_${item.key}", permIds)
//        }
//    }

}
