package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.plugins.Page
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysRoleMapper
import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.service.ISysPermissionService
import com.gxzc.zen.api.sys.service.ISysRoleService
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.orm.annotation.ZenTransactional
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysRoleServiceImpl : ServiceImpl<SysRoleMapper, SysRole>(), ISysRoleService {

    @Autowired
    private lateinit var sysPermissionService: ISysPermissionService

    @Autowired
    private lateinit var sysUserRoleService: ISysUserRoleService

    override fun getListByParam(name: String?, code: String?, enable: Boolean?): MutableList<SysRole> {
        val wrapper = EntityWrapper<SysRole>().apply {
            if (!name.isNullOrEmpty()) {
                like("name", name)
            }
            if (!code.isNullOrEmpty()) {
                like("code", code?.toUpperCase())
            }
            if (enable != null) {
                eq("enable", enable)
            }
        }

        return baseMapper.selectWOLogic(wrapper)
    }

    override fun getListByParamPage(name: String?, code: String?, enable: Boolean?, current: Int, size: Int): Page<SysRole> {
        val page = Page<SysRole>(current, size)

        val wrapper = EntityWrapper<SysRole>().apply {
            if (!name.isNullOrEmpty()) {
                like("name", name)
            }
            if (!code.isNullOrEmpty()) {
                like("code", code?.toUpperCase())
            }
            if (enable != null) {
                eq("enable", enable)
            }
        }
        page.records = baseMapper.selectWOLogicPage(page, wrapper)
        return page
    }

    @ZenTransactional
    override fun modify(entity: SysRole): SysRole {
//        baseMapper.updateById(entity)
        baseMapper.updateWOLogic(entity, EntityWrapper<SysRole>().eq("id", entity.id))

        val userIdSet = sysUserRoleService.getUserIdListByRoleId(entity.id!!)
        sysUserRoleService.evictCache(userIdSet)
        // 修改role表需要直接清理所有缓存项
//        sysPermissionService.evictCache(userIdSet)
        return entity
    }

    @ZenTransactional
    override fun physicalDelete(id: Long) {
        val userIdSet = sysUserRoleService.getUserIdListByRoleId(id)
        if (baseMapper.physicalDeleteById(id) <= 0) {
            throw ZenException(ZenExceptionEnum.BIZ_DELETE_ERROR)
        }
        // 修改role表需要直接清理所有缓存项
        sysUserRoleService.evictCache(userIdSet)
    }
}
