package com.gxzc.zen.api.sys.service.impl

import com.baomidou.mybatisplus.mapper.EntityWrapper
import com.baomidou.mybatisplus.service.impl.ServiceImpl
import com.gxzc.zen.api.sys.mapper.SysUserRoleMapper
import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.model.SysUserRole
import com.gxzc.zen.api.sys.service.ISysPermissionService
import com.gxzc.zen.api.sys.service.ISysUserRoleService
import com.gxzc.zen.common.contants.CACHEKEYS
import com.gxzc.zen.common.exception.ZenException
import com.gxzc.zen.common.exception.ZenExceptionEnum
import com.gxzc.zen.common.util.RedisCacheUtil
import com.gxzc.zen.orm.annotation.DynamicDataSource
import com.gxzc.zen.orm.annotation.ZenTransactional
import com.gxzc.zen.orm.contants.DSKey
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.cache.annotation.Cacheable
import org.springframework.stereotype.Service

/**
 * <p>
 * 用户角色表 服务实现类
 * </p>
 *
 * @author NorthLan123
 * @since 2018-02-06
 */
@Service
class SysUserRoleServiceImpl : ServiceImpl<SysUserRoleMapper, SysUserRole>(), ISysUserRoleService {

    @Autowired
    private lateinit var sysPermissionService: ISysPermissionService

    @Cacheable(CACHEKEYS.USER_ROLE, key = "'${CACHEKEYS.USER_ROLE}:uid_'+ #id")
    override fun getUserRoleListByUserId(id: Long): MutableList<SysRole> {
        return baseMapper.selectUserRoleListByUserId(id)
    }

    override fun getUserIdListByRoleId(id: Long): Set<Long> {
        val ret = baseMapper.selectList(EntityWrapper<SysUserRole>().eq("role_id", id)) ?: return setOf()
        return ret.map {
            it.userId!!
        }.toSet()
    }

    @ZenTransactional
    override fun physicalDeleteBatch(userId: Long, roleIds: List<Long>) {
        if (roleIds.isEmpty()) {
            return
        }
        baseMapper.physicalDelete(EntityWrapper<SysUserRole>()
                .eq("user_id", userId)
                .`in`("role_id", roleIds))
        evictCache(userId)
    }

    @ZenTransactional
    override fun physicalDeleteById(id: Long) {
        val tmp = baseMapper.selectWOLogic(EntityWrapper<SysUserRole>().eq("id", id))
        if (tmp.size < 1) {
            return
        }
        baseMapper.physicalDelete(EntityWrapper<SysUserRole>()
                .eq("id", id))
        evictCache(tmp[0].userId!!)
    }

    @ZenTransactional
    override fun physicalDeleteByUserId(userId: Long) {
        baseMapper.physicalDelete(EntityWrapper<SysUserRole>()
                .eq("user_id", userId))
        evictCache(userId)
    }

    @DynamicDataSource(DSKey.DSKEY_SYS)
    @ZenTransactional
    override fun insertBatch(userId: Long, roleIds: List<Long>) {
        if (roleIds.isEmpty()) {
            return
        }
        val dataList = mutableListOf<SysUserRole>()
        roleIds.forEach {
            dataList.add(SysUserRole(it, userId))
        }
        if (!this.insertBatch(dataList)) {
            throw ZenException(ZenExceptionEnum.BIZ_INSERT_ERROR)
        }
        evictCache(userId)
    }

    override fun evictCache(id: Long) {
        RedisCacheUtil.evict(CACHEKEYS.USER_ROLE, "${CACHEKEYS.USER_ROLE}:uid_$id")
        sysPermissionService.evictCache(id)
    }

    override fun evictCache(ids: Set<Long>) {
        ids.forEach {
            RedisCacheUtil.evict(CACHEKEYS.USER_ROLE, "${CACHEKEYS.USER_ROLE}:uid_$it")
            sysPermissionService.evictCache(it)
        }


    }
}
