package com.gxzc.zen.sso.aop

import com.gxzc.zen.api.util.SysUserUtil
import com.gxzc.zen.umps.util.SSOUtil
import org.slf4j.LoggerFactory
import org.springframework.web.method.HandlerMethod
import org.springframework.web.servlet.ModelAndView
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse

/**
 * 当前用户暂存拦截器
 * 在Controller方法执行前调用
 * @author NorthLan
 * @date 2018/3/24
 * @url https://noahlan.com
 */
class CurrentUserHandler : HandlerInterceptorAdapter() {
    companion object {
        private val logger = LoggerFactory.getLogger(CurrentUserHandler::class.java)
    }

    override fun preHandle(request: HttpServletRequest, response: HttpServletResponse, handler: Any): Boolean {
        if (handler is HandlerMethod) {
            val id = SSOUtil.getCurUserIdByToken(request)
            val user = if (id == null) {
                null
            } else {
                SysUserUtil.getById(id)
            }
            logger.trace("Caching Current User... ${user?.account}")
            SSOUtil.TL.set(user)
        }
        return super.preHandle(request, response, handler)
    }

    override fun postHandle(request: HttpServletRequest?, response: HttpServletResponse?, handler: Any?, modelAndView: ModelAndView?) {
        if (handler is HandlerMethod) {
            logger.trace("Clear Current User...")
            SSOUtil.TL.clear()
        }
        super.postHandle(request, response, handler, modelAndView)
    }
}