package com.gxzc.zen.umps.config

import com.baomidou.kisso.SSOConfig
import com.baomidou.kisso.SSOToken
import com.baomidou.kisso.annotation.Action
import com.baomidou.kisso.annotation.Permission
import com.baomidou.kisso.web.interceptor.SSOPermissionInterceptor
import org.springframework.web.method.HandlerMethod
import javax.servlet.http.HttpServletRequest


/**
 * 自定义权限验证 修改
 * 当无 @Permission 注解时可配置的跳过权限验证
 * @author NorthLan
 * @date 2018/3/10
 * @url https://noahlan.com
 */
class KissoPermissionInterceptor : SSOPermissionInterceptor() {
    var nothingAnnotationPass = false

    override fun isVerification(request: HttpServletRequest, handler: Any?, token: SSOToken?): Boolean {
        /*
         * URL 权限认证
		 */
        if (SSOConfig.getInstance().isPermissionUri) {
            val uri = request.requestURI
            if (uri == null || this.authorization.isPermitted(token, uri)) {
                return true
            }
        }
        /*
		 * 注解权限认证
		 */
        val handlerMethod = handler as HandlerMethod
        val method = handlerMethod.method
        val pm = method.getAnnotation(Permission::class.java)
        if (pm != null) {
            if (pm.action === Action.Skip) {
                /**
                 * 忽略拦截
                 */
                return true
            } else if ("" != pm.value && this.authorization.isPermitted(token, pm.value)) {
                /**
                 * 权限合法
                 */
                return true
            }
        } else if (this.nothingAnnotationPass) {
            /**
             * 无注解情况下，设置为true，不进行权限验证
             */
            return true
        }
        /*
		 * 非法访问
		 */
        return false
    }
}