package com.gxzc.zen.common.base

import com.baomidou.mybatisplus.mapper.BaseMapper
import com.baomidou.mybatisplus.mapper.Wrapper
import com.baomidou.mybatisplus.plugins.pagination.Pagination
import org.apache.ibatis.annotations.Param
import java.io.Serializable

/**
 * 通用的DAO
 * 提供一些通用的特殊查询方法(除了Mybatis-plus提供的自动crud)
 * @author NorthLan
 * @date 2018/1/24
 * @url https://noahlan.com
 */
interface BaseMapper<T> : BaseMapper<T> {
//    fun testSelect(@Param("ew") wrapper: Wrapper<T>): MutableList<T>

    /**
     * 通过自定义SQL查询
     * 忽略 全局 逻辑删除
     */
    fun selectWOLogic(@Param("ew") wrapper: Wrapper<T>?): MutableList<T>

    /**
     * 自定义SQL查询 分页
     * 忽略 全局 逻辑删除
     */
    fun selectWOLogicPage(page: Pagination, @Param("ew") wrapper: Wrapper<T>?): MutableList<T>

    /**
     * 自定义更新逻辑 更新enable字段时使用
     * 忽略 全局 逻辑删除
     * 自动生成sql,无需在xml中写入sql
     */
    fun updateWOLogic(@Param("et") entity: T, @Param("ew") wrapper: Wrapper<T>): Long

    /**
     * 物理删除
     * 自动生成sql,无需在xml中写入sql
     */
    fun physicalDelete(@Param("ew") wrapper: Wrapper<T>): Long

    /**
     * 物理删除 By ID
     * 自动生成sql,无需在xml中写入sql
     */
    fun physicalDeleteById(id: Serializable): Long
}