package com.gxzc.zen.common.config.cache.redis

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.cache.CacheManager
import org.springframework.cache.annotation.EnableCaching
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.data.redis.cache.RedisCacheManager
import org.springframework.data.redis.core.RedisTemplate
import org.springframework.data.redis.serializer.GenericJackson2JsonRedisSerializer
import org.springframework.data.redis.serializer.StringRedisSerializer

/**
 * Redis 缓存配置
 * 缓存管理器
 * @author NorthLan
 * @date 2018/3/29
 * @url https://noahlan.com
 */
@Configuration
@EnableCaching
@ConditionalOnProperty(prefix = "spring.cache", name = ["type"], havingValue = "redis")
class RedisConfig {

    @Bean
    fun cacheManager(redisTemplate: RedisTemplate<Any, Any>): CacheManager {
        return RedisCacheManager(redisTemplate.apply {
            val stringRedisSerializer = StringRedisSerializer()
            keySerializer = stringRedisSerializer
            hashKeySerializer = stringRedisSerializer
            valueSerializer = GenericJackson2JsonRedisSerializer()
        })
    }
}