package com.gxzc.zen.common.config.request

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.gxzc.zen.common.config.request.annotation.KVTypeEnum
import com.gxzc.zen.common.config.request.annotation.ZenRequestTypes
import com.gxzc.zen.common.dto.RequestDto
import org.springframework.core.MethodParameter
import org.springframework.http.HttpInputMessage
import org.springframework.http.converter.HttpMessageConverter
import org.springframework.web.bind.annotation.ControllerAdvice
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdviceAdapter
import java.lang.reflect.Type
import java.util.*

/**
 *
 * @author NorthLan
 * @date 2018/3/16
 * @url https://noahlan.com
 */
@ControllerAdvice
class ZenRequestBodyAdvice : RequestBodyAdviceAdapter() {

    private val mapper = ObjectMapper().apply { configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false) }

    override fun supports(methodParameter: MethodParameter, targetType: Type, converterType: Class<out HttpMessageConverter<*>>): Boolean {
        return methodParameter.hasMethodAnnotation(ZenRequestTypes::class.java) && methodParameter.parameterType == RequestDto::class.java
    }

    override fun afterBodyRead(body: Any, inputMessage: HttpInputMessage, parameter: MethodParameter, targetType: Type, converterType: Class<out HttpMessageConverter<*>>): Any {
        val annotation = parameter.getMethodAnnotation(ZenRequestTypes::class.java)
        if (annotation.value.isEmpty()) {
            return super.afterBodyRead(body, inputMessage, parameter, targetType, converterType)
        }
        val typedBody = body as RequestDto
        annotation.value.forEach {
            if (typedBody.containsKey(it.key)) {
                val data = typedBody[it.key]
                when (it.type) {
                    KVTypeEnum.OBJECT -> {
                        typedBody[it.key] = mapper.convertValue(data, it.value.java)
                    }
                    KVTypeEnum.ARRAY -> {
                        typedBody[it.key] = mapper.convertValue(data, mapper.typeFactory.constructArrayType(it.value.java))
                    }
                    KVTypeEnum.LIST -> {
                        typedBody[it.key] = mapper.convertValue(data, mapper.typeFactory.constructCollectionType(LinkedList::class.java, it.value.java))
                    }
                }
            }
        }
        return super.afterBodyRead(body, inputMessage, parameter, targetType, converterType)
    }

}