package com.gxzc.zen.common.config.response

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.ObjectMapper
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import kotlin.reflect.KClass

/**
 * 自定义的Json序列化工具类
 * @author NorthLan
 * @date 2018/3/12
 * @url https://noahlan.com
 */
class ZenJsonSerializer {
    val mapper = ObjectMapper().apply { configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false) }
    val jacksonFilter = ZenJacksonJsonFilter()

    fun filter(clazz: KClass<*>?, include: Array<String>?, filter: Array<String>?) {
        if (clazz == null) {
            return
        }
        if (include != null && include.isNotEmpty()) {
            jacksonFilter.include(clazz, include)
        }
        if (filter != null && filter.isNotEmpty()) {
            jacksonFilter.filter(clazz, filter)
        }
        mapper.addMixIn(clazz.java, ZenDynamicFilterMixin::class.java)
    }

    fun filter(zenResponseFilter: ZenResponseFilter) {
        this.filter(zenResponseFilter.type, zenResponseFilter.include, zenResponseFilter.filter)
    }

//    fun toJson(any: Any): String {
//        mapper.setFilterProvider(jacksonFilter)
//        return mapper.writeValueAsString(any)
//    }
}