package com.gxzc.zen.common.util

import net.sourceforge.pinyin4j.PinyinHelper
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination
import org.slf4j.LoggerFactory

/**
 * 拼音工具类
 * @author NorthLan at 2018/2/8
 */
object PinyinUtil {
    private val logger = LoggerFactory.getLogger(PinyinUtil::class.java)

    /**
     * 将中文转换为 全拼
     * 非汉字字符原样拼接
     * @param src 中文字符串
     * @param camel 是否输出驼峰格式化
     */
    fun convertToPinyin(src: String?, camel: Boolean = false): String {
        if (src.isNullOrEmpty()) {
            return ""
        }
        // 设置汉字拼音输出的格式
        val format = HanyuPinyinOutputFormat().apply {
            caseType = HanyuPinyinCaseType.LOWERCASE // 全小写
            toneType = HanyuPinyinToneType.WITHOUT_TONE // 无音标
            vCharType = HanyuPinyinVCharType.WITH_V // u->v
        }
        return try {
            var result = ""
            var temp: String
            for (ch in src!!) {
                var isChinese = false
                temp = if (ch.toString().matches(Regex("[\\u4E00-\\u9FA5]+"))) {
                    isChinese = true
                    val py = PinyinHelper.toHanyuPinyinStringArray(ch, format)
                    py[0] // 只取出第一种读音
                } else {
                    ch.toString()
                }
                if (camel && isChinese) {
                    temp = temp.substring(0, 1).toUpperCase() + temp.substring(1)
                }
                result += temp
            }
            result
        } catch (e: BadHanyuPinyinOutputFormatCombination) {
            logger.error("拼音转换错误", e)
            ""
        }
    }

    /**
     * 获取中文拼音首字母(小写)
     * @param src 中文字符串
     */
    fun getPinYinHead(src: String?): String {
        if (src.isNullOrEmpty()) {
            return ""
        }
        var result = ""
        for (ch in src!!) {
            if (ch.toString().matches(Regex("[\\u4E00-\\u9FA5]+"))) {
                val py = PinyinHelper.toHanyuPinyinStringArray(ch)
                result += if (py != null) {
                    py[0][0]
                } else {
                    ch.toString()
                }
            }
        }
        return result
    }
}