package com.gxzc.zen.common.util

import com.gxzc.zen.common.config.cache.redis.RedisKeyGenerator
import org.springframework.cache.Cache
import org.springframework.cache.CacheManager
import kotlin.reflect.KClass

/**
 * 缓存工具类 适用于 Redis
 * @author NorthLan at 2018/2/8
 */
object RedisCacheUtil {
    private val cacheManager = SpringContextHolder.getBean(CacheManager::class.java)

    fun getCahce(key: String): Cache? {
        return cacheManager.getCache(key)
    }

    /**
     * 获取指定类型缓存值
     */
    fun <T : Any> get(cacheKey: String, key: String, type: KClass<T>): T? {
        return getCahce(cacheKey)?.get(RedisKeyGenerator.joinToString(cacheKey, key), type.java)
    }

    /**
     * 获取ValueWrapper
     */
    fun get(cacheKey: String, key: String): Cache.ValueWrapper? {
        return getCahce(cacheKey)?.get(RedisKeyGenerator.joinToString(cacheKey, key))
    }

    /**
     * 插入(强制)
     */
    fun put(cacheKey: String, key: String, value: Any) {
        getCahce(cacheKey)?.put(RedisKeyGenerator.joinToString(cacheKey, key), value)
    }

    /**
     * 插入
     */
    fun putIfAbsent(cacheKey: String, key: String, value: Any) {
        getCahce(cacheKey)?.putIfAbsent(RedisKeyGenerator.joinToString(cacheKey, key), value)
    }

    /**
     * 移除缓存
     */
    fun evict(cacheKey: String, key: String) {
        getCahce(cacheKey)?.evict(key)
    }

    /**
     * 清除缓存
     */
    fun clear(cacheKey: String) {
        getCahce(cacheKey)?.clear()
    }
}