package com.gxzc.zen.orm.config

import com.alibaba.druid.pool.xa.DruidXADataSource
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean
import com.baomidou.mybatisplus.spring.boot.starter.MybatisPlusProperties
import com.baomidou.mybatisplus.spring.boot.starter.SpringBootVFS
import com.gxzc.zen.orm.DynamicMultipleDataSource
import com.gxzc.zen.orm.contants.DSKey
import org.apache.ibatis.plugin.Interceptor
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.ObjectProvider
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.boot.jta.atomikos.AtomikosDataSourceBean
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.DependsOn
import org.springframework.context.annotation.Primary
import javax.sql.DataSource
import javax.sql.XADataSource


/**
 *
 * @author NorthLan at 2018/1/29
 */
@Configuration
// @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
class MultipleDataSourceConfig {
    companion object {
        private val logger = LoggerFactory.getLogger(MultipleDataSourceConfig::class.java)
    }

    init {
        logger.info("MultipleDataSourceConfig initializing...")
    }

    @Bean(DSKey.DSKEY_SYS + "druid")
    @ConfigurationProperties(prefix = "datasource.sys")
    fun dataSourceSysDruid(): XADataSource {
        return DruidXADataSource()
    }

    @Bean(DSKey.DSKEY_BUSINESS + "druid")
    @ConfigurationProperties(prefix = "datasource.bus")
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun dataSourceBusDruid(): XADataSource {
        return DruidXADataSource()
    }

    @Bean(DSKey.DSKEY_SHARED + "druid")
    @ConfigurationProperties(prefix = "datasource.shared")
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun dataSourceSharedDruid(): XADataSource {
        return DruidXADataSource()
    }

    @Bean(DSKey.DSKEY_SYS)
    @DependsOn(DSKey.DSKEY_SYS + "druid")
    @Primary
    fun dataSourceSys(): DataSource {
        return AtomikosDataSourceBean().also {
            it.xaDataSource = dataSourceSysDruid()
        }
    }

    @Bean(DSKey.DSKEY_BUSINESS)
    @DependsOn(DSKey.DSKEY_BUSINESS + "druid")
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun dataSourceBus(): DataSource {
        return AtomikosDataSourceBean().also {
            it.xaDataSource = dataSourceBusDruid()
        }
    }

    @Bean(DSKey.DSKEY_SHARED)
    @DependsOn(DSKey.DSKEY_SHARED + "druid")
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun dataSourceShared(): DataSource {
        return AtomikosDataSourceBean().also {
            it.xaDataSource = dataSourceSharedDruid()
        }
    }

    @Bean
    @ConditionalOnProperty(prefix = "orm", name = ["multi-datasource-enable"], havingValue = "true", matchIfMissing = true)
    fun multipleDataSource(@Qualifier(DSKey.DSKEY_SYS) dataSourceSys: DataSource,
                           @Qualifier(DSKey.DSKEY_BUSINESS) dataSourceBus: DataSource,
                           @Qualifier(DSKey.DSKEY_SHARED) dataSourceShared: DataSource): DynamicMultipleDataSource {
        return DynamicMultipleDataSource().apply {
            setTargetDataSources(mapOf(
                    DSKey.DSKEY_SYS to dataSourceSys,
                    DSKey.DSKEY_BUSINESS to dataSourceBus,
                    DSKey.DSKEY_SHARED to dataSourceShared))
            setDefaultTargetDataSource(dataSourceSys)
        }
    }
}