package com.gxzc.zen.orm.sql

import com.baomidou.mybatisplus.entity.TableInfo
import com.baomidou.mybatisplus.mapper.LogicSqlInjector
import com.baomidou.mybatisplus.toolkit.StringUtils
import org.apache.ibatis.builder.MapperBuilderAssistant
import org.apache.ibatis.session.Configuration
import org.springframework.stereotype.Component


/**
 * 自定义 injector 继承自 LogicSqlInjector 同时实现逻辑删注入
 * 为几个公共自定义方法提供 sql 构建
 *
 * updateWOLogic
 * physicalDelete
 *
 * @author NorthLan
 * @date 2018/3/30
 * @url https://noahlan.com
 */
@Component
class ZenSqlInjector : LogicSqlInjector() {

    override fun inject(configuration: Configuration, builderAssistant: MapperBuilderAssistant, mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        super.inject(configuration, builderAssistant, mapperClass, modelClass, table)
//        this.testSelect(mapperClass, modelClass, table)
        this.updateWOLogic(mapperClass, modelClass, table)
        this.physicalDeleteById(mapperClass, modelClass, table)
        this.physicalDelete(mapperClass, modelClass, table)
        this.selectWOLogic(mapperClass, modelClass, table)
        this.selectWOLogicPage(mapperClass, modelClass, table)
    }

//    fun testSelect(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
//        val sqlMethod = "testSelect"
//        val sql = "<script>SELECT ${sqlSelectColumns(table, true)} FROM ${table.tableName} ${zenSqlWhereEntityWrapper(table)}</script>"
//        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
//        this.addSelectMappedStatement(mapperClass, sqlMethod, sqlSource, modelClass, table)
//    }

    fun updateWOLogic(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        val sqlMethod = "updateWOLogic"
        val sql = "<script>UPDATE ${table.tableName} ${sqlSet(true, table, "et.")} ${zenSqlWhereEntityWrapper(table)}</script>"
        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
        this.addUpdateMappedStatement(mapperClass, modelClass, sqlMethod, sqlSource)
    }

    fun physicalDeleteById(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        val sqlMethod = "physicalDeleteById"
        val sql = "<script>DELETE FROM ${table.tableName} WHERE ${table.keyColumn} = #{${table.keyProperty}}</script>"
        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
        this.addDeleteMappedStatement(mapperClass, sqlMethod, sqlSource)
    }

    fun physicalDelete(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        val sqlMethod = "physicalDelete"
        val sql = "<script>DELETE FROM ${table.tableName} ${zenSqlWhereEntityWrapper(table)}</script>"
        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
        this.addDeleteMappedStatement(mapperClass, sqlMethod, sqlSource)
    }

    fun selectWOLogic(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        val sqlMethod = "selectWOLogic"
        val sql = "<script>SELECT ${sqlSelectColumns(table, true)} FROM ${table.tableName} ${zenSqlWhereEntityWrapper(table)}</script>"
        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
        this.addSelectMappedStatement(mapperClass, sqlMethod, sqlSource, modelClass, table)
    }

    fun selectWOLogicPage(mapperClass: Class<*>, modelClass: Class<*>, table: TableInfo) {
        val sqlMethod = "selectWOLogicPage"
        val sql = "<script>SELECT ${sqlSelectColumns(table, true)} FROM ${table.tableName} ${zenSqlWhereEntityWrapper(table)}</script>"
        val sqlSource = languageDriver.createSqlSource(configuration, sql, modelClass)
        this.addSelectMappedStatement(mapperClass, sqlMethod, sqlSource, modelClass, table)
    }

    fun zenSqlWhereEntityWrapper(table: TableInfo): String {
        val where = StringBuilder(128)
        where.append("\n<where>")
        where.append("\n<if test=\"ew!=null\">")
        where.append("\n<if test=\"ew.entity!=null\">")
        if (StringUtils.isNotEmpty(table.keyProperty)) {
            where.append("\n<if test=\"ew.entity.").append(table.keyProperty).append("!=null\">\n")
            where.append(table.keyColumn).append("=#{ew.entity.").append(table.keyProperty).append("}")
            where.append("\n</if>")
        }
        val fieldList = table.fieldList
        for (fieldInfo in fieldList) {
            where.append(convertIfTag(fieldInfo, "ew.entity.", false))
            where.append(" AND ").append(this.sqlCondition(fieldInfo.condition,
                    fieldInfo.column, "ew.entity." + fieldInfo.el))
            where.append(convertIfTag(fieldInfo, true))
        }
        where.append("\n</if>")
        where.append("\n<if test=\"ew!=null and ew.sqlSegment!=null and ew.notEmptyOfWhere\">\n\${ew.sqlSegment}\n</if>")
        where.append("\n</if>")
        where.append("\n</where>")
        where.append("\n<if test=\"ew!=null and ew.sqlSegment!=null and ew.emptyOfWhere\">\n\${ew.sqlSegment}\n</if>")
        return where.toString()
    }
}