package com.gxzc.zen.web.sys.controller

import com.gxzc.zen.api.sys.model.SysRole
import com.gxzc.zen.api.sys.service.ISysRoleService
import com.gxzc.zen.common.base.BaseController
import com.gxzc.zen.common.config.response.annotation.ZenResponseFilter
import com.gxzc.zen.common.dto.ResponseDto
import com.gxzc.zen.common.util.PaginationUtil
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.net.URI

/**
 * 角色 控制器
 * @author NorthLan
 * @date 2018/3/24
 * @url https://noahlan.com
 */
@RestController
@RequestMapping("role")
class RoleController : BaseController() {
    companion object {
        private val logger = LoggerFactory.getLogger(RoleController::class.java)
    }

    @Autowired
    private lateinit var roleService: ISysRoleService

    @GetMapping("list")
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "createTime", "updateBy", "updateTime"])
    fun list(@RequestParam(required = false) keyword: String?,
             @RequestParam(required = false) searchOption: Int?,
             @RequestParam(required = false) enable: Boolean?): ResponseEntity<*> {
        var result: Any? = null
        if (!keyword.isNullOrEmpty() && searchOption != null) {
            when (searchOption) {
                1 -> {
                    result = if (PaginationUtil.paginable(getRequest())) {
                        roleService.getListByParamPage(keyword, null, enable, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
                    } else {
                        roleService.getListByParam(keyword, null, enable)
                    }
                }
                2 -> {
                    result = if (PaginationUtil.paginable(getRequest())) {
                        roleService.getListByParamPage(null, keyword, enable, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
                    } else {
                        roleService.getListByParam(null, keyword, enable)
                    }
                }
                else -> {
                    // 未知的查询关键字类型
                    logger.warn("Unknown searchOption: [$searchOption]")
                }
            }
        } else {
            result = if (PaginationUtil.paginable(getRequest())) {
                roleService.getListByParamPage(null, null, enable, PaginationUtil.getCurrent(getRequest())!!, PaginationUtil.getPageSize(getRequest())!!)
            } else {
                roleService.getListByParam(null, null, enable)
            }
        }
        return ResponseEntity.ok(ResponseDto().apply { data = result })
    }

    @GetMapping("{id}")
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun getById(@PathVariable id: Long): ResponseEntity<*> {
        return ResponseEntity.ok(ResponseDto().apply {
            data = roleService.selectById(id)
        })
    }

    @PutMapping
    @ZenResponseFilter(type = SysRole::class, filter = ["createBy", "updateBy", "createTime", "updateTime"])
    fun putDic(@RequestBody data: SysRole): ResponseEntity<*> {
        return if (data.id == null) {
            // insert
            roleService.insert(data)
            ResponseEntity.created(URI.create("/sys/dic/${data.id}")).body(ResponseDto()) // 201
        } else {
            // update
            ResponseEntity.ok(ResponseDto().apply {
                this.data = roleService.modify(data) // 200
            })
        }
    }

    @DeleteMapping("{id}")
    fun deleteDic(@PathVariable id: Long): ResponseEntity<*> {
        // 物理删除数据
        roleService.physicalDelete(id)
        return ResponseEntity.ok(ResponseDto())
    }
}